# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-22 04:37
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0017_auto_20180120_1700'),
    ]

    operations = [
        migrations.AddField(
            model_name='userexamrecord',
            name='cert',
            field=models.FilePathField(blank=True, null=True, verbose_name='\u8bc1\u4e66\u5730\u5740'),
        ),
    ]
