# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-20 09:00
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0016_auto_20180120_1605'),
    ]

    operations = [
        migrations.AlterField(
            model_name='gradecertfield',
            name='text_bg_color',
            field=models.CharField(blank=True, max_length=7, null=True, verbose_name='\u6587\u5b57\u533a\u57df\u80cc\u666f\u989c\u8272'),
        ),
    ]
