# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-03-27 07:22
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0022_auto_20180317_1613'),
    ]

    operations = [
        migrations.AddField(
            model_name='userexamrecord',
            name='status',
            field=models.IntegerField(blank=True, null=True, verbose_name='\u72b6\u6001'),
        ),
    ]
