# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-03-17 08:12
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0020_auto_20180126_1410'),
    ]

    operations = [
        migrations.AlterField(
            model_name='notice',
            name='context',
            field=models.TextField(null=True, verbose_name='\u987b\u77e5\u5185\u5bb9'),
        ),
    ]
