# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2018-01-14 07:51
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('bee_django_exam', '0006_auto_20180114_1418'),
    ]

    operations = [
        migrations.AddField(
            model_name='userexamrecord',
            name='passed_date',
            field=models.DateField(null=True, verbose_name='\u901a\u8fc7\u65e5\u671f'),
        ),
    ]
